{% load i18n %}
{{ message_recipient.message.text | safe }}

<hr>

<p>
    {% blocktrans with name=site.name %}Show this message on site {{ name }}:{% endblocktrans %}<br/>
    <a href="{{ site.url }}{{ message_recipient.get_absolute_url }}">
        {{ site.url }}{{ message_recipient.get_absolute_url }}
    </a>
</p>
