{% extends "admin/base_site.html" %}
{% load i18n l10n admin_urls %}

{% block content %}
<form method="post">{% csrf_token %}
{% for obj in queryset %}
<input type="hidden" name="{{ action_checkbox_name }}" value="{{ obj.pk|unlocalize }}" />
{% endfor %}
<input type="hidden" name="action" value="add_to_message" />
<input type="hidden" name="post" value="yes" />
{{ form.as_p }}
<input type="submit" value="{% trans 'add' %}" />
<a href="#" onclick="window.history.back(); return false;" class="button cancel-link">{% trans 'back' %}</a>
</form>
{% endblock %}
