{% load i18n %}

{{ object.text_registration_received }}

{% if object.subject.full %}
<p>
{% blocktrans with subject_type=object.subject.subject_type.name_genitiv %}Unfortunately, the capacity of this {{ subject_type }} has already been filled.
The registration may be refused.{% endblocktrans %}
</p>
{% endif %}

{% include 'leprikon/registration_mail_details.html' %}

{% include 'leprikon/registration_mail_footer.html' %}
